@@ -25,6 +25,7 @@
 #include "gnome-cmd-file-selector.h"
 #include "gnome-cmd-dir.h"
 #include "gnome-cmd-con.h"
+#include "gnome-cmd-con-list.h"
 #include "gnome-cmd-con-smb.h"
 #include "gnome-cmd-combo.h"
 #include "gnome-cmd-data.h"
@@ -87,15 +88,31 @@ static guint file_selector_signals[LAST_SIGNAL] = { 0 };
  * Utility functions
  *******************************/
 
-inline void set_same_directory (GnomeCmdFileSelector *fs, GnomeCmdFileSelector *other)
-{
-    GnomeCmdCon *con = gnome_cmd_file_selector_get_connection (other);
-    GnomeCmdDir *dir = gnome_cmd_file_selector_get_directory (other);
 
-    if (fs->priv->con != con)
-        gnome_cmd_file_selector_set_connection (fs, con, dir);
+void gnome_cmd_file_selector_set_same_directory (GnomeCmdMainWin *mw, FileSelectorID fsID, FileSelectorID otherID)
+{
+    g_return_if_fail (mw!=NULL);
+    g_return_if_fail (mw->priv!=NULL);
+
+    GnomeCmdFileSelector *fs = gnome_cmd_main_win_get_fs (mw, fsID);
+    GnomeCmdFileSelector *other = gnome_cmd_main_win_get_fs (mw, !fsID);
+
+    gboolean fs_is_active = fs->priv->active;
+
+    if (fs_is_active)
+        gnome_cmd_file_selector_set_directory (fs, gnome_cmd_file_selector_get_directory (other));
     else
-        gnome_cmd_file_selector_set_directory (fs, dir);
+    {
+        GnomeCmdFile *file = gnome_cmd_file_list_get_selected_file (other->list);
+
+        if (file && file->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+            gnome_cmd_file_selector_goto_directory (fs, gnome_cmd_file_get_real_path (file));
+        else
+            gnome_cmd_file_selector_set_directory (fs, gnome_cmd_file_selector_get_directory (other));
+    }
+
+    gnome_cmd_file_selector_set_active (other, !fs_is_active);
+    gnome_cmd_file_selector_set_active (fs, fs_is_active);
 }
 
 
@@ -223,7 +240,7 @@ inline void show_dir_tree_sizes (GnomeCmdFileSelector *fs)
 }
 
 
-static void set_connection (GnomeCmdFileSelector *fs, GnomeCmdCon *con, GnomeCmdDir *dir)
+inline void set_connection (GnomeCmdFileSelector *fs, GnomeCmdCon *con, GnomeCmdDir *dir=NULL)
 {
     fs->priv->con = con;
     fs->priv->dir_history = gnome_cmd_con_get_dir_history (con);
@@ -293,7 +310,7 @@ static gboolean file_is_wanted (GnomeCmdFileSelector *fs, GnomeVFSFileInfo *info
 }
 
 
-static GnomeCmdFile *create_parent_dir_file (GnomeCmdDir *dir)
+inline GnomeCmdFile *create_parent_dir_file (GnomeCmdDir *dir)
 {
     GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
 
@@ -307,8 +324,7 @@ static GnomeCmdFile *create_parent_dir_file (GnomeCmdDir *dir)
                                                    GNOME_VFS_FILE_INFO_FIELDS_SIZE |
                                                    GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE);
 
-    GnomeCmdFile *finfo = gnome_cmd_file_new (info, dir);
-    return finfo;
+    return gnome_cmd_file_new (info, dir);
 }
 
 
@@ -599,12 +615,7 @@ static void autoscroll_if_appropriate (GnomeCmdFileSelector *fs, gint x, gint y)
 }
 
 
-static gboolean
-drag_motion (GtkWidget *widget,
-             GdkDragContext *context,
-             gint x, gint y,
-             guint time,
-             GnomeCmdFileSelector *fs)
+static gboolean drag_motion (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time, GnomeCmdFileSelector *fs)
 {
     gdk_drag_status (context, context->suggested_action, time);
 
@@ -643,7 +654,7 @@ static void init_dnd (GnomeCmdFileSelector *fs)
     // Set up drag destination
     gtk_drag_dest_set (GTK_WIDGET (fs->list),
                        GTK_DEST_DEFAULT_DROP,
-                       drop_types, ELEMENTS (drop_types),
+                       drop_types, G_N_ELEMENTS (drop_types),
                        (GdkDragAction) (GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK));
 
     gtk_signal_connect (GTK_OBJECT (fs->list), "drag_motion", GTK_SIGNAL_FUNC (drag_motion), fs);
@@ -723,8 +734,8 @@ static void goto_directory (GnomeCmdFileSelector *fs, const gchar *in_dir)
     cur_dir = gnome_cmd_file_selector_get_directory (fs);
     g_return_if_fail (GNOME_CMD_IS_DIR (cur_dir));
 
-    if (strcmp (in_dir, "~") == 0)
-        dir = g_strdup (g_get_home_dir ());
+    if (g_str_has_prefix (in_dir, "~"))
+        dir = gnome_vfs_expand_initial_tilde (in_dir);
     else
         dir = unquote_if_needed (in_dir);
 
@@ -743,17 +754,16 @@ static void goto_directory (GnomeCmdFileSelector *fs, const gchar *in_dir)
     {
         // check if it's an absolute address or not
         if (dir[0] == '/')
-        {
             new_dir = gnome_cmd_dir_new (fs->priv->con, gnome_cmd_con_create_path (fs->priv->con, dir));
-        }
-        else if (strncmp (dir, "\\\\", 2) == 0)
-        {
-            GnomeCmdPath *path = gnome_cmd_con_create_path (get_smb_con (), dir);
-            if (path)
-                new_dir = gnome_cmd_dir_new (get_smb_con (), path);
-        }
         else
-            new_dir = gnome_cmd_dir_get_child (cur_dir, dir);
+            if (g_str_has_prefix (dir, "\\\\") )
+            {
+                GnomeCmdPath *path = gnome_cmd_con_create_path (get_smb_con (), dir);
+                if (path)
+                    new_dir = gnome_cmd_dir_new (get_smb_con (), path);
+            }
+            else
+                new_dir = gnome_cmd_dir_get_child (cur_dir, dir);
     }
 
     if (new_dir)
@@ -791,7 +801,7 @@ inline gboolean file_is_in_list (GnomeCmdFileSelector *fs, GnomeCmdFile *finfo)
 }
 
 
-static void add_file_to_cmdline (GnomeCmdFileSelector *fs, gboolean fullpath)
+inline void add_file_to_cmdline (GnomeCmdFileSelector *fs, gboolean fullpath)
 {
     g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 
@@ -877,13 +887,31 @@ static void on_dir_file_changed (GnomeCmdDir *dir, GnomeCmdFile *finfo, GnomeCmd
 }
 
 
+static void on_dir_file_renamed (GnomeCmdDir *dir, GnomeCmdFile *finfo, GnomeCmdFileSelector *fs)
+{
+    g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+    g_return_if_fail (GNOME_CMD_IS_FILE (finfo));
+    g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
+
+    if (file_is_in_list (fs, finfo))
+    {
+        gnome_cmd_file_list_update_file (GNOME_CMD_FILE_LIST (fs->list), finfo);
+
+        GnomeCmdFileListColumnID sort_col = gnome_cmd_file_list_get_sort_column (GNOME_CMD_FILE_LIST (fs->list));
+
+        if (sort_col==FILE_LIST_COLUMN_NAME || sort_col==FILE_LIST_COLUMN_EXT)
+            gnome_cmd_file_list_sort (GNOME_CMD_FILE_LIST (fs->list));
+    }
+}
+
+
 static void on_con_combo_item_selected (GnomeCmdCombo *con_combo, GnomeCmdCon *con, GnomeCmdFileSelector *fs)
 {
     g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
     g_return_if_fail (GNOME_CMD_IS_CON (con));
 
     gnome_cmd_main_win_switch_fs (main_win, fs);
-    gnome_cmd_file_selector_set_connection (fs, con, NULL);
+    gnome_cmd_file_selector_set_connection (fs, con);
 }
 
 
@@ -905,7 +933,7 @@ static void on_con_btn_clicked (GtkButton *button, GnomeCmdFileSelector *fs)
     g_return_if_fail (GNOME_CMD_IS_CON (con));
 
     gnome_cmd_main_win_switch_fs (main_win, fs);
-    gnome_cmd_file_selector_set_connection (fs, con, NULL);
+    gnome_cmd_file_selector_set_connection (fs, con);
 }
 
 
@@ -925,7 +953,7 @@ static void create_con_buttons (GnomeCmdFileSelector *fs)
     if (!tooltips)
         tooltips = gtk_tooltips_new ();
 
-    for (GList *l=gnome_cmd_con_list_get_all (gnome_cmd_data_get_con_list ()); l; l=l->next)
+    for (GList *l=gnome_cmd_con_list_get_all (gnome_cmd_con_list_get ()); l; l=l->next)
     {
         GtkWidget *btn, *label;
         GtkWidget *hbox;
@@ -1049,11 +1077,14 @@ static void on_dir_list_ok (GnomeCmdDir *dir, GList *files, GnomeCmdFileSelector
                                            GTK_SIGNAL_FUNC (on_dir_file_deleted), fs);
             gtk_signal_disconnect_by_func (GTK_OBJECT (fs->priv->connected_dir),
                                            GTK_SIGNAL_FUNC (on_dir_file_changed), fs);
+            gtk_signal_disconnect_by_func (GTK_OBJECT (fs->priv->connected_dir),
+                                           GTK_SIGNAL_FUNC (on_dir_file_renamed), fs);
         }
 
         gtk_signal_connect (GTK_OBJECT (dir), "file-created", GTK_SIGNAL_FUNC (on_dir_file_created), fs);
         gtk_signal_connect (GTK_OBJECT (dir), "file-deleted", GTK_SIGNAL_FUNC (on_dir_file_deleted), fs);
         gtk_signal_connect (GTK_OBJECT (dir), "file-changed", GTK_SIGNAL_FUNC (on_dir_file_changed), fs);
+        gtk_signal_connect (GTK_OBJECT (dir), "file-renamed", GTK_SIGNAL_FUNC (on_dir_file_renamed), fs);
         fs->priv->connected_dir = dir;
     }
 
@@ -1097,7 +1128,7 @@ static void on_dir_list_ok (GnomeCmdDir *dir, GList *files, GnomeCmdFileSelector
 static gboolean set_home_connection (GnomeCmdFileSelector *fs)
 {
     g_printerr ("Setting home connection\n");
-    gnome_cmd_file_selector_set_connection (fs, gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ()), NULL);
+    gnome_cmd_file_selector_set_connection (fs, get_home_con ());
 
     return FALSE;
 }
@@ -1109,7 +1140,7 @@ static void on_dir_list_failed (GnomeCmdDir *dir, GnomeVFSResult result, GnomeCm
 
     if (result != GNOME_VFS_OK)
     {
-        gchar *msg = g_strdup_printf (_("List failed: %s\n"), gnome_vfs_result_to_string (result));
+        gchar *msg = g_strdup_printf (_("Listing failed: %s\n"), gnome_vfs_result_to_string (result));
         create_error_dialog (msg);
         g_free (msg);
     }
@@ -1159,9 +1190,9 @@ static gboolean on_list_key_pressed_private (GtkCList *clist, GdkEventKey *event
 {
     if (state_is_blank (event->state) || state_is_shift (event->state))
     {
-        if ((event->keyval >= GDK_A && event->keyval <= GDK_Z)
-            || (event->keyval >= GDK_a && event->keyval <= GDK_z)
-            || event->keyval == GDK_period)
+        if ((event->keyval >= GDK_A && event->keyval <= GDK_Z) ||
+            (event->keyval >= GDK_a && event->keyval <= GDK_z) ||
+            event->keyval == GDK_period)
         {
             static gchar text[2];
 
@@ -1364,7 +1395,7 @@ static void init (GnomeCmdFileSelector *fs)
     gtk_signal_connect (GTK_OBJECT (fs->root_btn), "clicked",
                         GTK_SIGNAL_FUNC (on_root_btn_clicked), fs);
 
-    gtk_signal_connect (GTK_OBJECT (gnome_cmd_data_get_con_list ()), "list-changed",
+    gtk_signal_connect (GTK_OBJECT (gnome_cmd_con_list_get ()), "list-changed",
                         GTK_SIGNAL_FUNC (on_con_list_list_changed), fs);
 
 
@@ -1451,18 +1482,13 @@ void gnome_cmd_file_selector_start_editor (GnomeCmdFileSelector *fs)
         return;
 
     // create a command with an empty argument to the editor
-    gchar *cmd = g_strdup (gnome_cmd_data_get_editor ());
-    gint l = strlen(cmd);
-    for (gint i=0; i<l; i++)
-        if (cmd[i] == ' ')
-        {
-            cmd[i] = '\0';
-            break;
-        }
-
+    gchar *cmd = g_strdup_printf (gnome_cmd_data_get_editor (), "");
     gchar *dpath = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (fs->priv->cwd));
+
     run_command_indir (cmd, dpath, FALSE);
+
     g_free (dpath);
+    g_free (cmd);
 }
 
 
@@ -1645,7 +1671,7 @@ static void on_con_open_done (GnomeCmdCon *con, GnomeCmdFileSelector *fs)
     DEBUG('m', "on_con_open_done\n");
     gtk_signal_disconnect_by_data (GTK_OBJECT (con), fs);
 
-    set_connection (fs, con, NULL);
+    set_connection (fs, con);
 
     gtk_widget_destroy (fs->priv->con_open_dialog);
     fs->priv->con_open_dialog = NULL;
@@ -1713,7 +1739,7 @@ static void create_con_open_progress_dialog (GnomeCmdFileSelector *fs)
 }
 
 
-void gnome_cmd_file_selector_set_connection (GnomeCmdFileSelector *fs, GnomeCmdCon *con, GnomeCmdDir *dir)
+void gnome_cmd_file_selector_set_connection (GnomeCmdFileSelector *fs, GnomeCmdCon *con, GnomeCmdDir *start_dir)
 {
     g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
     g_return_if_fail (GNOME_CMD_IS_CON (con));
@@ -1737,7 +1763,7 @@ void gnome_cmd_file_selector_set_connection (GnomeCmdFileSelector *fs, GnomeCmdC
         gnome_cmd_con_open (con);
     }
     else
-        set_connection (fs, con, dir);
+        set_connection (fs, con, start_dir);
 }
 
 
@@ -1758,7 +1784,7 @@ void gnome_cmd_file_selector_update_connections (GnomeCmdFileSelector *fs)
     gtk_clist_set_row_height (GTK_CLIST (GNOME_CMD_COMBO (fs->con_combo)->list), 20);
     gtk_clist_set_column_width (GTK_CLIST (GNOME_CMD_COMBO (fs->con_combo)->list), 0, 20);
 
-    GnomeCmdConList *con_list = gnome_cmd_data_get_con_list ();
+    GnomeCmdConList *con_list = gnome_cmd_con_list_get ();
 
     for (GList *l=gnome_cmd_con_list_get_all (con_list); l; l = l->next)
     {
@@ -1789,7 +1815,7 @@ void gnome_cmd_file_selector_update_connections (GnomeCmdFileSelector *fs)
 
     // If the connection is no longer available use the home connection
     if (!found_my_con)
-        gnome_cmd_file_selector_set_connection (fs, gnome_cmd_con_list_get_home (con_list), NULL);
+        gnome_cmd_file_selector_set_connection (fs, get_home_con ());
     else
         gnome_cmd_combo_select_data (GNOME_CMD_COMBO (fs->con_combo), fs->priv->con);
 
@@ -1943,20 +1969,12 @@ gboolean gnome_cmd_file_selector_keypressed (GnomeCmdFileSelector *fs, GdkEventK
             case GDK_KP_Enter:
                 add_file_to_cmdline (fs, TRUE);
                 return TRUE;
-
-            case GDK_greater:
-                set_same_directory (gnome_cmd_main_win_get_fs (main_win, INACTIVE), fs);
-                return TRUE;
         }
     }
     else if (state_is_shift (event->state))
     {
         switch (event->keyval)
         {
-            case GDK_F4:
-                gnome_cmd_file_selector_start_editor (fs);
-                return TRUE;
-
             case GDK_F5:
                 gnome_cmd_file_list_show_make_copy_dialog (fs);
                 return TRUE;
@@ -2019,10 +2037,6 @@ gboolean gnome_cmd_file_selector_keypressed (GnomeCmdFileSelector *fs, GdkEventK
             case GDK_KP_Enter:
                 add_file_to_cmdline (fs, FALSE);
                 return TRUE;
-
-            case GDK_period:
-                set_same_directory (fs, gnome_cmd_main_win_get_fs (main_win, INACTIVE));
-                return TRUE;
         }
     }
     else if (state_is_blank (event->state))
