@@ -1,34 +1,37 @@
 /*
-  GNOME Commander - A GNOME based file manager
-  Copyright (C) 2001-2006 Marcus Bjurman
+    GNOME Commander - A GNOME based file manager
+    Copyright (C) 2001-2006 Marcus Bjurman
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include <config.h>
 #include <gtk/gtkclipboard.h>
+#include <algorithm>
 
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-bookmark-dialog.h"
 #include "gnome-cmd-con.h"
+#include "gnome-cmd-con-list.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-dir.h"
 #include "gnome-cmd-file-list.h"
 #include "gnome-cmd-file-selector.h"
 #include "gnome-cmd-file.h"
-#include "gnome-cmd-ftp-dialog.h"
+#include "gnome-cmd-con-dialog.h"
+#include "gnome-cmd-remote-dialog.h"
 #include "gnome-cmd-main-win.h"
 #include "gnome-cmd-options-dialog.h"
 #include "gnome-cmd-prepare-copy-dialog.h"
@@ -112,16 +115,28 @@ inline bool operator < (const GdkEventKey &e1, const GdkEventKey &e2)
 }
 
 
+inline gboolean ascii_isalnum (guint key_val)
+{
+    return key_val<=G_MAXUINT8 && g_ascii_isalnum (key_val);
+}
+
+
+inline gboolean ascii_isalpha (guint key_val)
+{
+    return key_val<=G_MAXUINT8 && g_ascii_isalpha (key_val);
+}
+
+
 inline string key2str(guint state, guint key_val)
 {
     string key_name;
 
-    if (state & GDK_SHIFT_MASK)    key_name += "<shift>";
-    if (state & GDK_CONTROL_MASK)  key_name += "<control>";
-    if (state & GDK_MOD1_MASK)     key_name += "<alt>";
-    if (state & GDK_MOD4_MASK)     key_name += "<win>";
+    if (state & GDK_SHIFT_MASK)    key_name += gdk_modifiers_names[GDK_SHIFT_MASK];
+    if (state & GDK_CONTROL_MASK)  key_name += gdk_modifiers_names[GDK_CONTROL_MASK];
+    if (state & GDK_MOD1_MASK)     key_name += gdk_modifiers_names[GDK_MOD1_MASK];
+    if (state & GDK_MOD4_MASK)     key_name += gdk_modifiers_names[GDK_MOD4_MASK];
 
-    if (g_ascii_isalnum (key_val))
+    if (ascii_isalnum (key_val))
         key_name += g_ascii_tolower (key_val);
     else
         key_name += gdk_key_names[key_val];
@@ -147,16 +162,22 @@ inline GdkEventKey str2key(gchar *s, guint &state, guint &key_val)
     state = 0;
 
      if (key_val==GDK_VoidSymbol)
-        if (strlen(key)==1 && g_ascii_isalnum (*key))
+        if (strlen(key)==1 && ascii_isalnum (*key))
             key_val = *key;
 
-    if (strstr (s, "<shift>"))    state |= GDK_SHIFT_MASK;
-    if (strstr (s, "<control>"))  state |= GDK_CONTROL_MASK;
-    if (strstr (s, "<alt>"))      state |= GDK_MOD1_MASK;
-    if (strstr (s, "<win>"))      state |= GDK_MOD4_MASK;
+    for (const gchar *beg=s; (beg=strchr(beg, '<')); ++beg)
+    {
+        if (const gchar *end = strchr(beg, '>'))
+            if (guint modifier = gdk_modifiers_names[string(beg,end-beg+1)])
+            {
+                state |= modifier;
+                beg = end;
+                continue;
+            }
 
-    if (strstr (s, "<mod1>"))      state |= GDK_MOD1_MASK;
-    if (strstr (s, "<mod4>"))      state |= GDK_MOD4_MASK;
+        key_val = GDK_VoidSymbol;
+        break;
+    }
 
     GdkEventKey event;
 
@@ -186,10 +207,10 @@ void GnomeCmdUserActions::init()
     actions.add(bookmarks_add_current, "bookmarks.add_current");
     actions.add(bookmarks_edit, "bookmarks.edit");
     actions.add(bookmarks_goto, "bookmarks.goto");
+    actions.add(command_open_terminal, "command.open_terminal");
     actions.add(connections_close_current, "connections.close");
-    actions.add(connections_close_current, "connections.close_current");
-    actions.add(connections_ftp_connect, "connections.ftp_connect");
-    actions.add(connections_ftp_quick_connect, "connections.ftp_quick_connect");
+    actions.add(connections_new, "connections.new");
+    actions.add(connections_open, "connections.open");
     actions.add(edit_cap_copy, "edit.copy");
     actions.add(edit_copy_fnames, "edit.copy_filenames");
     actions.add(edit_cap_cut, "edit.cut");
@@ -206,6 +227,7 @@ void GnomeCmdUserActions::init()
     actions.add(file_delete, "file.delete");
     actions.add(file_diff, "file.diff");
     actions.add(file_edit, "file.edit");
+    actions.add(file_edit_new_doc, "file.edit_new_doc");
     actions.add(file_exit, "file.exit");
     actions.add(file_external_view, "file.external_view");
     actions.add(file_internal_view, "file.internal_view");
@@ -214,6 +236,7 @@ void GnomeCmdUserActions::init()
     actions.add(file_properties, "file.properties");
     actions.add(file_rename, "file.rename");
     // actions.add(file_run, "file.run");
+    actions.add(file_sendto, "file.sendto");
     actions.add(file_sync_dirs, "file.synchronize_directories");
     // actions.add(file_umount, "file.umount");
     actions.add(file_view, "file.view");
@@ -236,12 +259,18 @@ void GnomeCmdUserActions::init()
     actions.add(view_equal_panes, "view.equal_panes");
     actions.add(view_first, "view.first");
     actions.add(view_forward, "view.forward");
+    actions.add(view_home, "view.home");
+    actions.add(view_in_active_pane, "view.in_active_pane");
+    actions.add(view_in_inactive_pane, "view.in_inactive_pane");
+    actions.add(view_in_left_pane, "view.in_left_pane");
+    actions.add(view_in_right_pane, "view.in_right_pane");
     actions.add(view_last, "view.last");
     actions.add(view_refresh, "view.refresh");
+    actions.add(view_root, "view.root");
     actions.add(view_up, "view.up");
 
     register_action(GDK_F3, "file.view");
-    register_action(GDK_F4, "file.mkdir");
+    register_action(GDK_F4, "file.edit");
     register_action(GDK_F5, "file.copy");
     register_action(GDK_F6, "file.rename");
     register_action(GDK_F7, "file.mkdir");
@@ -254,14 +283,14 @@ void GnomeCmdUserActions::init()
     if (!registered("bookmarks.edit"))
         register_action(GDK_CONTROL_MASK, GDK_D, "bookmarks.edit");
 
-    if (!registered("connections.close_current"))
-        register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_F, "connections.close_current");
+    if (!registered("connections.new"))
+        register_action(GDK_CONTROL_MASK, GDK_N, "connections.new");
 
-    if (!registered("connections.ftp_connect"))
-        register_action(GDK_CONTROL_MASK, GDK_F, "connections.ftp_connect");
+    if (!registered("connections.open"))
+        register_action(GDK_CONTROL_MASK, GDK_F, "connections.open");
 
-    if (!registered("connections.ftp_quick_connect"))
-        register_action(GDK_CONTROL_MASK, GDK_G, "connections.ftp_quick_connect");
+    if (!registered("connections.close"))
+        register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_F, "connections.close");
 
     if (!registered("edit.copy_filenames"))
         register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_C, "edit.copy_filenames");
@@ -281,6 +310,9 @@ void GnomeCmdUserActions::init()
     if (!registered("file.create_symlink"))
         register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_F5, "file.create_symlink");
 
+    if (!registered("file.edit_new_doc"))
+        register_action(GDK_SHIFT_MASK, GDK_F4, "file.edit_new_doc");
+
     if (!registered("file.external_view"))
         register_action(GDK_MOD1_MASK, GDK_F3, "file.external_view");
 
@@ -310,6 +342,39 @@ void GnomeCmdUserActions::init()
         register_action(GDK_CONTROL_MASK, GDK_KP_5, "plugins.execute_python", "md5sum");
     }
 
+    if (!registered("view.equal_panes"))
+    {
+        register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_equal, "view.equal_panes");
+        register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_plus, "view.equal_panes");
+    }
+
+    if (!registered("view.in_active_pane"))
+        register_action(GDK_CONTROL_MASK, GDK_period, "view.in_active_pane");
+
+    if (!registered("view.in_inactive_pane"))
+        register_action(GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_greater, "view.in_inactive_pane");
+
+    if (!registered("view.in_left_pane"))
+    {
+        register_action(GDK_CONTROL_MASK, GDK_Left, "view.in_left_pane");
+        register_action(GDK_CONTROL_MASK, GDK_KP_Left, "view.in_left_pane");
+    }
+
+    if (!registered("view.in_right_pane"))
+    {
+        register_action(GDK_CONTROL_MASK, GDK_Right, "view.in_right_pane");
+        register_action(GDK_CONTROL_MASK, GDK_KP_Right, "view.in_right_pane");
+    }
+
+    if (!registered("view.home"))
+    {
+        register_action(GDK_CONTROL_MASK, GDK_quoteleft, "view.home");
+        register_action(GDK_CONTROL_MASK, GDK_asciitilde, "view.home");
+    }
+
+    if (!registered("view.root"))
+        register_action(GDK_CONTROL_MASK, GDK_backslash, "view.root");
+
     if (!registered("view.refresh"))
         register_action(GDK_CONTROL_MASK, GDK_R, "view.refresh");
  }
@@ -391,9 +456,9 @@ void GnomeCmdUserActions::write(const gchar *section)
     for (gpointer i=gnome_config_init_iterator (section_path.c_str()); (i=gnome_config_iterator_next (i, &key, &action_name)); )
     {
         string curr_key = key;
-        string norm_key = key2str(str2key(key));    // <ALT><Ctrl>F3 -> <ctrl><alt>f3
+        string norm_key = key2str(str2key(key));        // <ALT><Ctrl>F3 -> <ctrl><alt>f3
 
-        if (curr_key!=norm_key)
+        if (!norm_key.empty() && curr_key!=norm_key)    // if (norm_key is valid && not 'canonical')
         {
             gnome_config_clean_key ((section_path+curr_key).c_str());
             gnome_config_set_string ((section_path+norm_key).c_str(), action_name);
@@ -435,7 +500,7 @@ gboolean GnomeCmdUserActions::register_action(guint state, guint keyval, const g
     if (action.find(event)!=action.end())
         return FALSE;
 
-    if (!g_ascii_isalpha (keyval))
+    if (!ascii_isalpha (keyval))
         action[event] = UserAction(func, user_data);
     else
     {
@@ -449,6 +514,11 @@ gboolean GnomeCmdUserActions::register_action(guint state, guint keyval, const g
 }
 
 
+void GnomeCmdUserActions::unregister(const gchar *name)
+{
+}
+
+
 void GnomeCmdUserActions::unregister(guint state, guint keyval)
 {
     GdkEventKey event;
@@ -573,6 +643,12 @@ void file_edit (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+void file_edit_new_doc (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_start_editor (get_fs (ACTIVE));
+}
+
+
 void file_chmod (GtkMenuItem *menuitem, gpointer not_used)
 {
     gnome_cmd_file_list_show_chmod_dialog (get_fl (ACTIVE));
@@ -631,6 +707,34 @@ void file_advrename (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+void file_sendto (GtkMenuItem *menuitem, gpointer not_used)
+{
+    string cmd = "nautilus-sendto";
+
+    GnomeCmdFileList *fl = get_fl (ACTIVE);
+    GList *sfl = gnome_cmd_file_list_get_selected_files (fl);
+    sfl = gnome_cmd_file_list_sort_selection (sfl, fl);
+
+    for (GList *i = sfl; i; i = i->next)
+    {
+        GnomeCmdFile *finfo = GNOME_CMD_FILE (i->data);
+
+        if (!finfo)
+            continue;
+
+        cmd += ' ';
+        cmd += (char *) gnome_cmd_file_get_quoted_real_path (finfo);
+    }
+
+    g_list_free (sfl);
+
+    g_print (_("running `%s'\n"), cmd.c_str());
+
+    if (cmd != "nautilus-sendto")
+        run_command (cmd.c_str(), FALSE);
+}
+
+
 void file_properties (GtkMenuItem *menuitem, gpointer not_used)
 {
     gnome_cmd_file_list_show_properties_dialog (get_fl (ACTIVE));
@@ -824,6 +928,16 @@ void edit_copy_fnames (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+/************** Command Menu **************/
+void command_open_terminal (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gchar *dpath = gnome_cmd_file_get_real_path (GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (get_fs (ACTIVE))));
+
+    gnome_execute_terminal_shell (dpath, NULL);
+    g_free (dpath);
+}
+
+
 /************** Mark Menu **************/
 void mark_toggle (GtkMenuItem *menuitem, gpointer not_used)
 {
@@ -998,6 +1112,43 @@ void view_equal_panes (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+void view_in_left_pane (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), LEFT, RIGHT);
+}
+
+
+void view_in_right_pane (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), RIGHT, LEFT);
+}
+
+
+void view_in_active_pane (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), ACTIVE, INACTIVE);
+}
+
+
+void view_in_inactive_pane (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_set_same_directory ((GnomeCmdMainWin *) GTK_WIDGET (main_win), INACTIVE, ACTIVE);
+}
+
+
+void view_home (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_set_connection (get_fs (ACTIVE), get_home_con ());
+    gnome_cmd_file_selector_goto_directory (get_fs (ACTIVE), "~");
+}
+
+
+void view_root (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_selector_goto_directory (get_fs (ACTIVE), "/");
+}
+
+
 /************** Options Menu **************/
 void options_edit (GtkMenuItem *menuitem, gpointer not_used)
 {
@@ -1015,23 +1166,23 @@ void options_edit_mime_types (GtkMenuItem *menuitem, gpointer not_used)
 
 
 /************** Connections Menu **************/
-void connections_ftp_connect (GtkMenuItem *menuitem, gpointer not_used)
+void connections_open (GtkMenuItem *menuitem, gpointer not_used)
 {
-    GtkWidget *dialog = gnome_cmd_ftp_dialog_new ();
+    GtkWidget *dialog = gnome_cmd_remote_dialog_new ();
     gtk_widget_ref (dialog);
     gtk_widget_show (dialog);
 }
 
 
-void connections_ftp_quick_connect (GtkMenuItem *menuitem, gpointer not_used)
+void connections_new (GtkMenuItem *menuitem, gpointer not_used)
 {
-    show_ftp_quick_connect_dialog ();
+    show_quick_connect_dialog ();
 }
 
 
 void connections_change (GtkMenuItem *menuitem, gpointer con)
 {
-    gnome_cmd_file_selector_set_connection (get_fs (ACTIVE), (GnomeCmdCon *) con, NULL);
+    gnome_cmd_file_selector_set_connection (get_fs (ACTIVE), (GnomeCmdCon *) con);
 }
 
 
@@ -1042,12 +1193,12 @@ void connections_close (GtkMenuItem *menuitem, gpointer con)
 
     GnomeCmdCon *c1 = gnome_cmd_file_selector_get_connection (active);
     GnomeCmdCon *c2 = gnome_cmd_file_selector_get_connection (inactive);
-    GnomeCmdCon *home = gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ());
+    GnomeCmdCon *home = get_home_con ();
 
     if (con == c1)
-        gnome_cmd_file_selector_set_connection (active, home, NULL);
+        gnome_cmd_file_selector_set_connection (active, home);
     if (con == c2)
-        gnome_cmd_file_selector_set_connection (inactive, home, NULL);
+        gnome_cmd_file_selector_set_connection (inactive, home);
 
     gnome_cmd_con_close ((GnomeCmdCon *) con);
 }
@@ -1079,6 +1230,49 @@ void bookmarks_edit (GtkMenuItem *menuitem, gpointer not_used)
 
 void bookmarks_goto (GtkMenuItem *menuitem, gpointer bookmark_name)
 {
+    if (!bookmark_name)
+        return;
+
+    vector<string> a;
+    guint n = split(static_cast<gchar*>(bookmark_name), a, "|");
+    string group = n<2 || a[0].empty() ? "local" : a[0];
+    string name = a[n<2 ? 0 : 1];
+
+    transform(group.begin(), group.end(), group.begin(), (int(*)(int))tolower);
+
+    if (group=="local")
+    {
+        for (GList *bookmarks = gnome_cmd_con_get_bookmarks (get_home_con ())->bookmarks; bookmarks; bookmarks = bookmarks->next)
+        {
+            GnomeCmdBookmark *bookmark = (GnomeCmdBookmark *) bookmarks->data;
+
+            if (name == bookmark->name)
+            {
+                gnome_cmd_bookmark_goto (bookmark);
+                return;
+            }
+        }
+
+        g_warning ("[%s] Unknown bookmark name: '%s' - ignored", (char *) bookmark_name, name.c_str());
+    }
+    else
+        if (group=="smb" || group=="samba")
+        {
+            for (GList *bookmarks = gnome_cmd_con_get_bookmarks (get_smb_con ())->bookmarks; bookmarks; bookmarks = bookmarks->next)
+            {
+                GnomeCmdBookmark *bookmark = (GnomeCmdBookmark *) bookmarks->data;
+
+                if (name == bookmark->name)
+                {
+                    gnome_cmd_bookmark_goto (bookmark);
+                    return;
+                }
+            }
+
+            g_warning ("[%s] Unknown bookmark name: '%s' - ignored", (char *) bookmark_name, name.c_str());
+        }
+        else
+            g_warning ("[%s] Unsupported bookmark group: '%s' - ignored", (char *) bookmark_name, group.c_str());
 }
 
 
@@ -1116,13 +1310,13 @@ void plugins_execute_python (GtkMenuItem *menuitem, gpointer python_script)
 
 void help_help (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_help_display("gnome-commander.xml");
+    gnome_cmd_help_display ("gnome-commander.xml");
 }
 
 
 void help_keyboard (GtkMenuItem *menuitem, gpointer not_used)
 {
-    gnome_cmd_help_display("gnome-commander.xml","gnome-commander-keyboard");
+    gnome_cmd_help_display ("gnome-commander.xml","gnome-commander-keyboard");
 }
 
 
@@ -1130,8 +1324,8 @@ void help_web (GtkMenuItem *menuitem, gpointer not_used)
 {
     GError *error = NULL;
 
-    if (!gnome_url_show("http://www.nongnu.org/gcmd/", &error))
-        gnome_cmd_error_message(_("There was an error opening home page."), error);
+    if (!gnome_url_show ("http://www.nongnu.org/gcmd/", &error))
+        gnome_cmd_error_message (_("There was an error opening home page."), error);
 }
 
 
@@ -1140,7 +1334,7 @@ void help_problem (GtkMenuItem *menuitem, gpointer not_used)
     GError *error = NULL;
 
     if (!gnome_url_show("http://bugzilla.gnome.org/browse.cgi?product=gnome-commander", &error))
-        gnome_cmd_error_message(_("There was an error reporting problem."), error);
+        gnome_cmd_error_message (_("There was an error reporting problem."), error);
 }
 
 
@@ -1175,7 +1369,7 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
            "GNU General Public License for more details."),
         N_("You should have received a copy of the GNU General Public License "
            "along with GNOME Commander; if not, write to the Free Software Foundation, Inc., "
-           "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA")
+           "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.")
     };
 
     gchar *license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]), _(license[2]), NULL);
